@@ -1,12 +1,12 @@
 ;;; dicom.el --- DICOM viewer - Digital Imaging & Communications in Medicine -*- lexical-binding: t -*-
 
-;; Copyright (C) 2024-2026 Free Software Foundation, Inc.
+;; Copyright (C) 2024-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2024
-;; Package-Version: 1.3
-;; Package-Revision: 1.3-0-g03a12498905e
+;; Package-Version: 20251013.929
+;; Package-Revision: 74c8fa6a55c4
 ;; Package-Requires: ((emacs "29.1") (compat "30"))
 ;; URL: https://github.com/minad/dicom
 ;; Keywords: multimedia, hypermedia, files
@@ -38,11 +38,10 @@
 ;; image files interactively.
 
 ;; Emacs must be compiled with support for PNG, SVG and XML.  The package relies
-;; on external programs from the dcmtk DICOM toolkit, which are widely available
-;; on Linux distributions.
+;; on external programs from the dcmtk DICOM toolkit, which are all widely
+;; available on Linux distributions.
 
-;; - `dcm2xml' and `dcm2img' from the dcmtk DICOM toolkit
-;; - `magick' from ImageMagick
+;; - `dcm2xml' and `dcmj2pnm' from the dcmtk DICOM toolkit
 ;; - `ffmpeg' for video conversion (optional)
 ;; - `mpv' for video playing (optional)
 
@@ -67,7 +66,7 @@
   :group 'multimedia
   :prefix "dicom-")
 
-(defcustom dicom-timeout 5
+(defcustom dicom-timeout 3
   "Timeout for conversion."
   :type 'natnum)
 
@@ -76,11 +75,7 @@
   :type 'natnum)
 
 (defcustom dicom-attribute-width 25
-  "Attribute name width in characters."
-  :type 'natnum)
-
-(defcustom dicom-thumb-height 200
-  "Thumbnail height in pixels."
+  "Attribute name width."
   :type 'natnum)
 
 (defcustom dicom-attribute-filter
@@ -132,12 +127,10 @@ progress:${percent-pos}%%' %s) & disown"
 (defvar-keymap dicom-mode-map
   :doc "Keymap used by `dicom-mode'."
   :parent special-mode-map
-  "y" #'dicom-play
+  "p" #'dicom-play
   "+" #'dicom-larger
   "-" #'dicom-smaller
   "r" #'dicom-rotate
-  "p" #'dicom-previous
-  "n" #'dicom-next
   "TAB" #'outline-cycle
   "<backtab>" #'outline-cycle-buffer)
 
@@ -165,16 +158,19 @@ progress:${percent-pos}%%' %s) & disown"
 (defvar-local dicom--file nil
   "File associated with the current buffer.")
 
-(defvar-local dicom--image-buffer nil
-  "Image buffer associated with the current buffer.")
-
 (defvar-local dicom--queue nil
   "Conversion process queue in current buffer.")
 
 (defvar-local dicom--procs nil
   "Active conversion processes in current buffer.")
 
-(defvar dicom--thumb nil
+(defconst dicom--thumb
+  '( :margin 8 :type svg :width 267 :height 200
+     :data "<svg xmlns='http://www.w3.org/2000/svg' width='267' height='200'>
+  <rect width='267' height='200' fill='black' stroke='gray'/>
+  <line x1='0' y1='0' x2='267' y2='200' stroke='gray'/>
+  <line x1='0' y1='200' x2='267' y2='0' stroke='gray'/>
+</svg>")
   "Thumbnail placeholder image.")
 
 ;;;; Internal functions
@@ -193,7 +189,7 @@ progress:${percent-pos}%%' %s) & disown"
 
 (defun dicom--image-desc (file)
   "Image descriptor for FILE."
-  `(image :margin 6 :type png :file ,file))
+  `(image :margin 8 :type png :file ,file))
 
 (defun dicom--dir-p (&optional file)
   "Non-nil if FILE is a DICOMDIR."
@@ -209,7 +205,7 @@ progress:${percent-pos}%%' %s) & disown"
              (directory-file-name
               (file-name-parent-directory file))))
     (list "dicom image: "
-          (if-let* ((dir (locate-dominating-file file "DICOMDIR")))
+          (if-let ((dir (locate-dominating-file file "DICOMDIR")))
               (file-name-sans-extension
                (file-relative-name file (file-name-parent-directory dir)))
             (file-name-base file)))))
@@ -243,28 +239,27 @@ progress:${percent-pos}%%' %s) & disown"
      (nconc (dicom--sort-alist (dicom--convert-children dom 'element))
             (dicom--sort-alist (dicom--convert-children dom 'sequence))))
     ('element
-     (when-let* ((name (dom-attr dom 'name))
-                 ((not (or (equal (dom-attr dom 'loaded) "no")
-                           (equal (dom-attr dom 'binary) "hidden")
-                           (let (case-fold-search)
-                             (string-match-p dicom-attribute-filter name))))))
+     (when-let ((name (dom-attr dom 'name))
+                ((not (or (equal (dom-attr dom 'loaded) "no")
+                          (equal (dom-attr dom 'binary) "hidden")
+                          (let (case-fold-search)
+                            (string-match-p dicom-attribute-filter name))))))
        (cons (intern name) (replace-regexp-in-string
                             "[ \t\n^]+" " " (with-no-warnings (dom-text dom))))))
     ('sequence
-     (when-let* ((name (dom-attr dom 'name))
-                 ((not (let (case-fold-search)
-                         (string-match-p dicom-attribute-filter name))))
-                 (children (dicom--convert-children dom)))
+     (when-let ((name (dom-attr dom 'name))
+                ((not (let (case-fold-search)
+                        (string-match-p dicom-attribute-filter name))))
+                (children (dicom--convert-children dom)))
        (cons (intern name) children)))))
 
 (defun dicom--read (file)
   "Read DICOM FILE and return list of items."
   (with-temp-buffer
-    (let ((coding-system-for-read 'latin-1)
-          (coding-system-for-write 'latin-1))
-      (unless (eq 0 (call-process "dcm2xml" nil t nil
-                                  "--quiet" "--charset-assume" "latin-1" file))
-        (error "DICOM: Reading DICOM metadata with dcm2xml failed")))
+    (unless (eq 0 (call-process "dcm2xml" nil t nil
+                                "--quiet" "--charset-assume"
+                                "latin-1" "--convert-to-utf8" file))
+      (error "DICOM: Reading DICOM metadata with dcm2xml failed"))
     (let ((dicom-attribute-filter (mapconcat (lambda (x) (format "%s" x))
                                              dicom-attribute-filter
                                              "\\|")))
@@ -272,38 +267,27 @@ progress:${percent-pos}%%' %s) & disown"
 
 (defun dicom--image-buffer ()
   "Return image buffer or throw an error."
-  (or (and dicom--file
-           (if (dicom--dir-p)
-               (and (buffer-live-p dicom--image-buffer)
-                    dicom--image-buffer)
-             (current-buffer)))
-      (user-error "DICOM: No open image")))
-
-(defun dicom--dir-buffer ()
-  "Return dir buffer or throw an error."
-  (or (and dicom--file
-           (if (dicom--dir-p)
-               (current-buffer)
-             (when-let* ((dir (locate-dominating-file dicom--file "DICOMDIR")))
-               (get-buffer (dicom--buffer-name (file-name-concat dir "DICOMDIR"))))))
-      (user-error "DICOM: No open DICOMDIR found")))
+  (if (dicom--dir-p)
+      (or (get-buffer "*dicom image*")
+          (user-error "DICOM: No open image"))
+    (current-buffer)))
 
 (defun dicom--modify-image (fun)
   "Modify image properties by FUN."
   (with-current-buffer (dicom--image-buffer)
-    (when-let* ((pos (text-property-not-all (point-min) (point-max) 'dicom--image nil))
-                (image (get-text-property pos 'display)))
+    (when-let ((pos (text-property-not-all (point-min) (point-max) 'dicom--image nil))
+               (image (get-text-property pos 'display)))
       (with-silent-modifications
         (goto-char pos)
         (funcall fun image)
         (put-text-property pos (1+ pos) 'display `(image ,@(cdr image)))))))
 
-(defun dicom--run (cb cmd)
-  "Run process with CMD asynchronously and call CB when the process finished."
+(defun dicom--run (cb &rest args)
+  "Run process with ARGS asynchronously and call CB when the process finished."
   (let ((default-directory "/"))
     (push (make-process
            :name "dicom"
-           :command (list "sh" "-c" cmd)
+           :command args
            :noquery t
            :filter #'ignore
            :sentinel
@@ -318,14 +302,9 @@ progress:${percent-pos}%%' %s) & disown"
     (when dicom-timeout
       (run-at-time dicom-timeout nil #'dicom--stop (car dicom--procs)))))
 
-(defun dicom--enqueue (cb fmt &rest args)
-  "Enqueue conversion job with callback CB.
-The command is specified as FMT string with ARGS."
-  (push (cons cb (apply #'format fmt
-                        (mapcar (lambda (x)
-                                  (if (numberp x) x (shell-quote-argument x)))
-                                args)))
-        dicom--queue)
+(defun dicom--enqueue (&rest job)
+  "Enqueue conversion JOB."
+  (push job dicom--queue)
   (when (length< dicom--procs dicom-parallel)
     (dicom--process)))
 
@@ -333,9 +312,9 @@ The command is specified as FMT string with ARGS."
   "Process conversion queue."
   (setq mode-line-process (and dicom--queue
                                (format "[%d]" (length dicom--queue))))
-  (when-let* ((job (car (last dicom--queue))))
+  (when-let ((job (car (last dicom--queue))))
     (setq dicom--queue (nbutlast dicom--queue))
-    (dicom--run (car job) (cdr job))))
+    (apply #'dicom--run job)))
 
 (defun dicom--button (label action)
   "Insert button with LABEL and ACTION."
@@ -385,15 +364,7 @@ The command is specified as FMT string with ARGS."
     (delete-region pos (point))
     (insert (propertize
              " "
-             'display (if exists
-                          (dicom--image-desc dst)
-                        (unless dicom--thumb
-                          (setq dicom--thumb
-                                (cdr (dicom--placeholder
-                                      (round (* 4 dicom-thumb-height) 3)
-                                      dicom-thumb-height))))
-                        `(image ,@dicom--thumb))
-             'cursor-face (list :inherit 'cursor) ;; New cons cell
+             'display (if exists (dicom--image-desc dst) `(image ,@dicom--thumb))
              'pointer 'hand
              'keymap dicom-image-map
              'dicom--file src
@@ -401,14 +372,7 @@ The command is specified as FMT string with ARGS."
     (unless exists
       (dicom--enqueue
        (dicom--image-callback tmp dst pos)
-       ;; If you wonder why we have to try these different commands to extract
-       ;; an image, maybe contribute patches to `dcm2img' or `magick' to improve
-       ;; coverage over the many DICOM image variants. Every order would work.
-       ;; The only difference might be performance. 😉
-       "dcm2img --write-png --scale-y-size %d %s %s || magick %s[0] -resize x%d %s || magick %s[-1] -resize x%d %s"
-       dicom-thumb-height src tmp
-       src dicom-thumb-height tmp
-       src dicom-thumb-height tmp))))
+       "dcmj2pnm" "--write-png" "--scale-y-size" "200" src tmp))))
 
 (defun dicom--item (level item &optional indent)
   "Insert ITEM at LEVEL into buffer."
@@ -420,14 +384,10 @@ The command is specified as FMT string with ARGS."
           (if (equal type "IMAGE")
               (dicom--thumb level item)
             (dicom--title level
-                          (format "%s %s %s" type
+                          (format "%s %s" type
                                   (or (alist-get 'StudyDescription item)
                                       (alist-get 'SeriesDescription item)
                                       (alist-get 'PatientName item)
-                                      "")
-                                  (or (alist-get 'StudyDate item)
-                                      (alist-get 'PatientBirthDate item)
-                                      (alist-get 'SeriesNumber item)
                                       "")))
             (dicom--item level item)))))
      ((listp v)
@@ -450,7 +410,7 @@ The command is specified as FMT string with ARGS."
 (defun dicom--placeholder (w h)
   "Placeholder image with W and H."
   `(image
-    :margin 6 :type svg :width ,w :height ,h
+    :margin 8 :type svg :width ,w :height ,h
     :data
     ,(format
       "<svg xmlns='http://www.w3.org/2000/svg' width='%1$s' height='%2$s'>
@@ -466,9 +426,7 @@ The command is specified as FMT string with ARGS."
   (dicom--button "Larger" #'dicom-larger)
   (dicom--button "Smaller" #'dicom-smaller)
   (dicom--button "Rotate" #'dicom-rotate)
-  (dicom--button "Previous" #'dicom-previous)
-  (dicom--button "Next" #'dicom-next)
-  (when-let* ((frames (alist-get 'NumberOfFrames dicom--data)))
+  (when-let ((frames (alist-get 'NumberOfFrames dicom--data)))
     (dicom--button (format "Play (%s frames)" frames) #'dicom-play))
   (insert "\n" (propertize "\n" 'face '(:height 0.2)))
   (pcase-let* ((`(,dst . ,tmp) (dicom--cache-name (concat "large" dicom--file)))
@@ -478,7 +436,6 @@ The command is specified as FMT string with ARGS."
              " "
              'dicom--image t
              'pointer 'arrow
-             'cursor-face 'cursor
              'display (if exists
                           (dicom--image-desc dst)
                         (dicom--placeholder
@@ -488,8 +445,7 @@ The command is specified as FMT string with ARGS."
     (unless exists
       (dicom--enqueue
        (dicom--image-callback tmp dst pos)
-       "dcm2img --write-png %s %s || magick %s[0] %s || magick %s[-1] %s"
-       dicom--file tmp dicom--file tmp dicom--file tmp))))
+       "dcmj2pnm" "--write-png" dicom--file tmp))))
 
 (defun dicom--setup-check ()
   "Check requirements."
@@ -501,7 +457,7 @@ The command is specified as FMT string with ARGS."
     (dolist (type '(png svg))
       (unless (image-type-available-p type)
         (push (format "lib%s" type) req)))
-    (dolist (exe '("dcm2xml" "dcm2img" "magick"))
+    (dolist (exe '("dcm2xml" "dcmj2pnm"))
       (unless (executable-find exe)
         (push exe req)))
     (when req
@@ -513,7 +469,6 @@ The command is specified as FMT string with ARGS."
               dicom--procs nil
               dicom--file file
               dicom--data (dicom--read file)
-              default-directory (file-name-directory dicom--file)
               buffer-read-only t
               truncate-lines nil
               word-wrap t
@@ -524,7 +479,6 @@ The command is specified as FMT string with ARGS."
               outline-regexp "\\*+"
               outline-minor-mode-cycle t
               outline-minor-mode-use-buttons 'in-margins
-              cursor-face-highlight-nonselected-window t
               header-line-format
               (format (propertize " DICOM %s %s" 'face 'dicom-header)
                       (if (dicom--dir-p) "DIR" "IMAGE")
@@ -546,12 +500,10 @@ The command is specified as FMT string with ARGS."
       (progn
         (dicom--setup-check)
         (mapc #'dicom--stop dicom--procs)
-        (rename-buffer (dicom--buffer-name file) t)
         (dicom-mode)
         (dicom--setup-locals file)
         (dicom--setup-content)
-        (outline-minor-mode)
-        (cursor-face-highlight-mode))
+        (outline-minor-mode))
     (error
      (kill-buffer)
      (signal (car err) (cdr err)))))
@@ -579,35 +531,6 @@ The command is specified as FMT string with ARGS."
   (interactive "p" dicom-mode)
   (dicom-larger (- n)))
 
-(defun dicom-next (&optional n)
-  "Go forward N images."
-  (interactive "p" dicom-mode)
-  (setq n (or n 1))
-  (with-current-buffer (dicom--dir-buffer)
-    (if (> n 0)
-        (while-let (((> n 0))
-                    (pt (next-single-property-change
-                         (point) 'dicom--file nil (point-max))))
-          (goto-char pt)
-          (when (or (eobp) (get-text-property pt 'dicom--file))
-            (cl-decf n)))
-      (while-let (((< n 0))
-                  (pt (previous-single-property-change
-                       (point) 'dicom--file nil (point-min))))
-        (goto-char pt)
-        (when (or (bobp) (get-text-property pt 'dicom--file))
-          (cl-incf n))))
-    (when-let* ((win (get-buffer-window)))
-      (set-window-point win (point)))
-    (dicom-open
-     (or (get-text-property (point) 'dicom--file)
-         (user-error "DICOM: No image found")))))
-
-(defun dicom-previous (&optional n)
-  "Go backward N images."
-  (interactive "p" dicom-mode)
-  (dicom-next (- (or n 1))))
-
 (defun dicom-play ()
   "Play DICOM multi frame image."
   (interactive nil dicom-mode)
@@ -633,51 +556,48 @@ The command is specified as FMT string with ARGS."
                    (rename-file tmp dst)
                    (dicom-play))
                (delete-file tmp)))
-           "(dcm2img --all-frames --write-bmp %s || magick %s bmp:-) | ffmpeg -framerate %s -i - %s"
-           dicom--file dicom--file
-           (or (alist-get 'RecommendedDisplayFrameRate dicom--data)
-               (alist-get 'CineRate dicom--data)
-               "25")
-           tmp)))))))
+           "sh" "-c"
+           (format
+            "dcmj2pnm --all-frames --write-bmp %s | ffmpeg -framerate %s -i - %s"
+            (shell-quote-argument dicom--file)
+            (or (alist-get 'RecommendedDisplayFrameRate dicom--data)
+                (alist-get 'CineRate dicom--data)
+                25)
+            (shell-quote-argument tmp)))))))))
 
 ;;;###autoload
 (defun dicom-open-at-point ()
   "Open DICOM at point."
   (interactive)
-  (if-let* ((file
-             (if (mouse-event-p last-input-event)
-                 (or (mouse-posn-property (event-start last-input-event)
-                                          'dicom--file)
-                     (thing-at-mouse last-input-event 'filename))
-               (or (get-text-property (point) 'dicom--file)
-                   (thing-at-point 'filename)))))
-      (dicom-open file)
+  (if-let ((file
+            (if (mouse-event-p last-input-event)
+                (or (mouse-posn-property (event-start last-input-event)
+                                         'dicom--file)
+                    (thing-at-mouse last-input-event 'filename))
+              (or (get-text-property (point) 'dicom--file)
+                  (thing-at-point 'filename)))))
+      (dicom-open file (and (not last-prefix-arg) "*dicom image*"))
     (user-error "DICOM: No DICOM file at point")))
 
 ;;;###autoload
-(defun dicom-open (file)
-  "Open DICOM dir or image FILE."
+(defun dicom-open (file &optional reuse)
+  "Open DICOM dir or image FILE.
+REUSE can be a buffer name to reuse."
   (interactive "fDICOM: ")
   (let* ((file (expand-file-name (if (directory-name-p file)
                                      (file-name-concat file "DICOMDIR")
                                    file)))
-         (_ (unless (file-regular-p file)
-              (user-error "DICOM: File %s not found" file)))
-         (reuse (and (buffer-live-p dicom--image-buffer) dicom--image-buffer))
-         (buf (or reuse (get-buffer-create (dicom--buffer-name file)))))
-    (unless (equal (buffer-local-value 'dicom--file buf) file)
-      (with-current-buffer buf
+         (default-directory (file-name-directory file))
+         (buf (or reuse (dicom--buffer-name file))))
+    (unless (file-regular-p file)
+      (user-error "DICOM: File %s not found" file))
+    (unless (when-let ((buf (get-buffer buf)))
+              (equal (buffer-local-value 'dicom--file buf) file))
+      (with-current-buffer (get-buffer-create buf)
         (dicom--setup file)))
-    (when (dicom--dir-p)
-      (setq dicom--image-buffer buf))
-    (cond
-     ((with-current-buffer (window-buffer)
-        (and dicom--file (not (dicom--dir-p))))
-      (switch-to-buffer buf nil t))
-     ((with-current-buffer (window-buffer) (dicom--dir-p))
-      (display-buffer buf '(nil (inhibit-same-window . t))))
-     (t
-      (pop-to-buffer buf '(nil (inhibit-same-window . t)))))))
+    (if reuse
+        (display-buffer buf '(nil (inhibit-same-window . t)))
+      (pop-to-buffer buf))))
 
 ;;;###autoload
 (defun dicom-bookmark-jump (bm)
@@ -689,14 +609,11 @@ The command is specified as FMT string with ARGS."
 ;;;###autoload
 (defun dicom-auto-mode ()
   "Enable `dicom-mode' in current buffer."
-  (let ((file buffer-file-name))
-    (if-let* ((buf (get-buffer (dicom--buffer-name file))))
-        (progn
-          (kill-current-buffer)
-          (switch-to-buffer buf))
-      (setq-local buffer-file-name nil
-                  buffer-file-truename nil)
-      (dicom--setup file))))
+  (let ((file (expand-file-name buffer-file-name)))
+    (setq-local buffer-file-name nil
+                buffer-file-truename nil)
+    (rename-buffer (dicom--buffer-name file) t)
+    (dicom--setup file)))
 
 ;;;###autoload
 (progn
